import typing

import aioimport

import yab.backend
import yab.data

if typing.TYPE_CHECKING:
    import discord


class DiscordPyContext(typing.NamedTuple):
    client: "discord.Client"
    message: "discord.Message"


class Discord(yab.backend.Backend):
    token: str

    def __init__(self, *, token: str) -> None:
        super().__init__()
        self.token = token

    async def main(
        self,
        protected_state: yab.backend.ProtectedState,
        queue: yab.backend.MessageQueue,
    ) -> None:
        await aioimport.import_module("discord")
        import discord  # noqa: E402

        client = discord.Client()

        class Send(yab.backend.Send):
            async def __call__(
                self, content: yab.data.Content, channel: yab.data.Channel,
            ) -> None:
                await client.get_channel(channel).send(content)

        @client.event
        async def on_ready() -> None:
            async with protected_state.lock_for_write() as state:
                state.can.send = Send()
                state.ready = True

        @client.event
        async def on_message(message: discord.Message) -> None:
            if message.author != client.user:
                await queue.put(
                    yab.data.Message(
                        content=message.content,
                        channel=message.channel.id,
                        env={
                            DiscordPyContext: DiscordPyContext(
                                client=client, message=message,
                            )
                        },
                    )
                )

        await client.start(self.token)
