import typing

import aioimport

import yab.backend
import yab.data
import yab.task

if typing.TYPE_CHECKING:
    import aiogram


class AIOGramContext(typing.NamedTuple):
    bot: "aiogram.Bot"
    dispatcher: "aiogram.Dispatcher"
    message: "aiogram.types.Message"


class Telegram(yab.backend.Backend):
    token: str

    def __init__(self, *, token: str) -> None:
        super().__init__()
        self.token = token

    async def main(
        self,
        protected_state: yab.backend.ProtectedState,
        queue: yab.backend.MessageQueue,
    ) -> None:
        await aioimport.import_module("aiogram")
        import aiogram  # noqa: E402

        bot = aiogram.Bot(token=self.token)
        dispatcher = aiogram.Dispatcher(bot)

        class Send(yab.backend.Send):
            async def __call__(
                self, content: yab.data.Content, channel: yab.data.Channel,
            ) -> None:
                await bot.send_message(channel, content)

        async def on_ready() -> None:
            async with protected_state.lock_for_write() as state:
                state.can.send = Send()
                state.ready = True

        @dispatcher.message_handler()
        async def on_message(message: aiogram.types.Message) -> None:
            await queue.put(
                yab.data.Message(
                    content=message.text,
                    channel=message.chat.id,
                    env={
                        AIOGramContext: AIOGramContext(
                            bot=bot, dispatcher=dispatcher, message=message,
                        ),
                    },
                )
            )

        async with yab.task.ContextManager(on_ready(), name="telegram backend ready"):
            await dispatcher.start_polling()
