import typing

import yab.data
import yab.middleware


PREFIX = "!"


class ParsedCommand(typing.NamedTuple):
    command: str
    reminder: str
    args: typing.Tuple[str, ...]


class CommandParser(yab.middleware.Middleware):
    async def apply(
        self, _, message: yab.data.Message
    ) -> typing.Optional[ParsedCommand]:
        text = message.content
        if text.startswith(PREFIX):
            command, *reminder = text.split(None, 1)
            return ParsedCommand(
                command=command[len(PREFIX) :],  # noqa: E203
                reminder=reminder[0] if reminder else "",
                args=(tuple(reminder[0].split()) if reminder else tuple()),
            )
        return None
