import asyncio
import collections
import typing

import yab.protected


_Value = typing.TypeVar("_Value")


class Queue(typing.Generic[_Value]):
    __deque: yab.protected.Protected[typing.Deque[_Value]]
    __event_clear: asyncio.Event

    def __init__(self, maxsize: typing.Optional[int] = None) -> None:
        self.__deque = yab.protected.Protected(collections.deque(maxlen=maxsize))
        self.__event_clear = asyncio.Event()

    async def get(self) -> _Value:
        while True:
            event_clear = self.__event_clear
            async with self.__deque.wait_for_update_and_lock() as deque:
                if event_clear.is_set():
                    continue
                if not deque:
                    continue
                return deque.popleft()

    async def put(self, value: _Value) -> None:
        event_clear = self.__event_clear
        async with self.__deque.lock_for_write() as deque:
            if event_clear.is_set():
                return
            deque.append(value)

    async def clear(self) -> None:
        async with self.__deque.lock_for_write() as deque:
            event_clear, self.__event_clear = (
                self.__event_clear,
                asyncio.Event(),
            )
            event_clear.set()
            deque.clear()
